// Depixelize
// Paulo Ricardo Reinehr
// PS 2.0
// Smoothen edges of color clusters similar to Eagle and 2xSaI, with scale factor and blending control. Work best with pixelart and flat shaded images.


//#PARAM percent blending 1 : Blending : Effect blending with the original upscaled image.
float blending;

//#PARAM float scale 2 : Scale : Scale Factor. Works best with integer values above 1. 2 gives maximum smoothness. Use 1 (no scale) for previously scaled images. 
float scale;


// Foreground texture
texture ForegroundTexture;

// Foreground sampler
sampler2D foreground = sampler_state {
    Texture = (ForegroundTexture);
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = Point;
};


float pixelWidth;
float pixelHeight;
float boxWidth;
float boxHeight;
float boxRight;
float boxBottom;


// Effect function
float4 EffectProcess( float2 Tex : TEXCOORD0 ) : COLOR0
{
	
// a   c
//   p  
// d   b

	scale = 1 / scale ;
	float2 xy = Tex.xy ;
	xy += float2( boxRight / ( 1 + boxWidth ), boxBottom / ( 1 + boxHeight ) ) / scale * ( 1 - scale ) ;
	xy *= scale ;


	float4 p = tex2D( foreground, xy );
	float alpha = p.a ;

	float4 a = tex2D( foreground, xy + float2( - pixelWidth , - pixelHeight ) * scale );
	float4 b = tex2D( foreground, xy + float2( + pixelWidth , + pixelHeight ) * scale );
	float4 c = tex2D( foreground, xy + float2( + pixelWidth , - pixelHeight ) * scale );
	float4 d = tex2D( foreground, xy + float2( - pixelWidth , + pixelHeight ) * scale );

	float4 a1 = abs( a - p );
	float4 b1 = abs( b - p );
	float4 c1 = abs( c - p );
	float4 d1 = abs( d - p );

	float a2 = a1.r + a1.g + a1.b ;
	float b2 = b1.r + b1.g + b1.b ;
	float c2 = c1.r + c1.g + c1.b ;
	float d2 = d1.r + d1.g + d1.b ;

	float4 x = 0;

	if ( a2 == b2 ){ x += (a + b) / 2; }else{ if ( a2 < b2 ){ x += a; }else{ x += b; }; };
	if ( c2 == d2 ){ x += (c + d) / 2; }else{ if ( c2 < d2 ){ x += c; }else{ x += d; }; };

	x /= 2 ;

	x *= blending ;
	x += p * ( 1 - blending ) ;
	x.a = alpha ;
	
	return x ;
	
}

// ConstructEffect
technique ConstructEffect
{
    pass p0
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 EffectProcess();
    }
}
